import networkx as nx
import osmnx as ox
import pandas as pd
import random
import math


from . import utils as u
from . import reliability as r



class Region:

    id_region = 0

    label_region = "region"
    regiontag_prefix = "cr.region-"

    def __init__(self, G, target_id = -1):
        self.G = G
        self.edges = []
        self.nodes = []
        if target_id == -1:
            self.id = Region.id_region
            Region.id_region += 1
        else:
            self.id = target_id
            if Region.id_region <= target_id:
                Region.id_region = target_id + 1

    def is_crossroad(self):
        return False

    def is_branch(self):
        return False

    def clear_region(self):
        # remove edges
        for e in self.edges:
            self.G[e[0]][e[1]][0][Region.label_region] = -1
        
        # then remove nodes
        for n in self.nodes:
            self.G.nodes[n][Region.label_region] = -1
    

    def init_attr(G):
        nx.set_edge_attributes(G, values=-1, name=Region.label_region)
        nx.set_node_attributes(G, values=-1, name=Region.label_region)

    def unknown_region_node_in_graph(G, n):
        return G.nodes[n][Region.label_region] == -1

    def unknown_region_edge_in_graph(G, e):
        return G[e[0]][e[1]][0][Region.label_region] == -1

    def unknown_region_node(self, n):
        return Region.unknown_region_node_in_graph(self.G, n)

    def unknown_region_edge(self, e):
        return Region.unknown_region_edge_in_graph(self.G, e)

    def clear_node_region_in_grah(G, n):
        G.nodes[n][Region.label_region] = -1

    def add_node(self, n):
        if n not in self.nodes:
            self.nodes.append(n)
            self.G.nodes[n][Region.label_region] = self.id

    def add_edge(self, e):
        if not self.has_edge(e):
            self.edges.append(e)
            self.G[e[0]][e[1]][0][Region.label_region] = self.id

    def add_path(self, path):
        for p in path:
            self.add_node(p)
        for p1, p2 in zip(path, path[1:]):
            self.add_edge((p1, p2))

    def has_edge(self, e):
        return e in self.edges or (e[1], e[0]) in self.edges

    def is_boundary_node(self, n):
        nbnb = len(list(self.G.neighbors(n)))
        nbEdgesInside = len([e for e in self.edges if e[0] == n or e[1] == n])
        return nbnb != nbEdgesInside

